@@ -126,7 +126,7 @@ def _gen(s: str):
     #
     # Parse seed file, which is in yaml format
     # e.g,
-    # template: apple
+    # template: template-apple
     # blueprints:
     #   - name: dashboard
     #     views:
@@ -138,15 +138,14 @@ def _gen(s: str):
     with open(seed_file) as stream:
         try:
             seed_content = yaml.safe_load(stream)
-            # TODO: Validate the seed content
         except yaml.YAMLError as e:
             logger.error(f'Can not parse seed file {seed_file}, {e}')
             return False
     #
-    # Get template name from seed file, and then use .name/seed name as the template folder, as one template can support many platforms
-    # e.g, app.sd using template apple -> .apple/app/
+    # Get template name from seed file, and then use name/seed name as the template folder, as one template can support many platforms
+    # e.g, app.sd using template apple -> apple/app/
     #
-    template_path = os.path.join('.' + seed_content['template'], s)
+    template_path = os.path.join(seed_content['template'], s)
     if not os.path.exists(template_path):
         # TODO: Download template to the folder
         logger.error(f'Can not find template folder {template_path}')
@@ -165,6 +164,7 @@ def _gen(s: str):
     #
     models = {}
     for m in registered_models:
+        logger.debug(f'Load model {m.__name__} with relations {list(m.__relations__.keys())}')
         models[m.__name__] = {'schema': m.schema(), **generate_names(m.__name__)}
     #
     logger.info(f'Found {len(models)} registered models: {list(models.keys())}')
@@ -182,7 +182,7 @@ def _gen(s: str):
     for bp in seed_content['blueprints']:  # Blueprints
         bp_name = bp['name']
         logger.info(f'{bp_name}/')
-        blueprint = {'views': [], **generate_names(bp_name)}
+        blueprint = {'views': [], 'params': bp.get('params', {}), **generate_names(bp_name)}
         models_by_name = {}
         for v in bp['views']:  # Views
             v_name = v['name']
@@ -203,13 +203,8 @@ def _gen(s: str):
                 seed_model = seed['model']
                 models_by_name[seed_model['name']] = seed_model
                 # Add relation models
-                # TODO: Replace old __relations__, so that we do not need to check very detail schema here
                 for relation in seed_model['schema']['relations']:
-                    relation_schema = seed_model['schema']['properties'][relation]
-                    relation_model = relation_schema['items'] if relation_schema['type'] == 'array' else relation_schema
-                    #
-                    relation_model_name = relation_model['py_type']
-                    models_by_name[relation_model_name] = models[relation_model_name]
+                    models_by_name[relation] = models[relation]
             #
             blueprint['views'].append(view)
             blueprint['models'] = models_by_name.values()
@@ -265,7 +260,9 @@ def _recursive_render(t_base, o_base, name, context, env):
             out_names = [t_name.replace(syntax, v['name']) for v in out_values]
         elif key == 'models':
             out_key = '__model'
-            out_values = list(context['models'].values())  # models can be accessed at context level, NOTE: models is dict, {name: {names, schema}}}, so we use values() here
+            # models can be accessed at context level, NOTE: models is dict, {name: {names, schema}}}, so we use values() here
+            out_values = list(context['models'].values())
+            # names of blueprints/views/seeds are kebab formats because them will be used in the url directly, while modal names are always in camel case because of PEP8
             out_names = [t_name.replace(syntax, v['name_kebab']) for v in out_values]
         else:
             raise TemplateError(f'Unsupported list syntax: {syntax}')
@@ -317,6 +314,7 @@ def _recursive_render(t_base, o_base, name, context, env):
     else:
         for o_name in out_names:
             o_path = os.path.join(o_base, o_name)
+            # TODO: Merge logic for non-template files, e.g, layout files such as demo.html
             logger.debug(f'copy {o_name}')
             shutil.copyfile(t_path, o_path)
             shutil.copymode(t_path, o_path)
@@ -329,7 +327,7 @@ def _recursive_render(t_base, o_base, name, context, env):
         o_base = os.path.abspath(o_base)
         with work_in(o_base):
             # Set jinja2's path
-            env.loader = FileSystemLoader(['.', os.path.join(t_base, 'includes')])  # search in current folder or template/includes folder
+            env.loader = FileSystemLoader('.')
             o_context = {k: v for k, v in context.items() if not k.startswith('__')}
             #
             for i, o_name in enumerate(out_names):
@@ -348,7 +346,7 @@ def _recursive_render(t_base, o_base, name, context, env):
                     exception.translated = False
                     raise
                 rendered = tmpl.render(**o_context)
-                #
+                # TODO: Merge logic for generated file
                 with open(o_file, 'w', encoding='utf-8') as f:
                     f.write(rendered)
                 # Remove template file
