@@ -8,7 +8,6 @@
     :copyright: (c) 2021 by weiminfeng.
     :date: 2021/6/5
 """
-import functools
 import json
 import sys
 from abc import ABCMeta
@@ -217,6 +216,8 @@ class ModelJSONEncoder(json.JSONEncoder):
             return str(o)
         elif isinstance(o, datetime):
             return o.strftime(DATETIME_FORMAT)
+        elif isinstance(o, BaseModel):
+            return o.dict()
 
         return json.JSONEncoder.default(self, o)
 
@@ -249,7 +250,7 @@ class ModelField:
         'type',
         'default',
         'required',
-        'readonly',
+        'editable',
         'searchable',
         'sortable',
         'format',
@@ -258,14 +259,15 @@ class ModelField:
         'description',
         'unit',
         'source_field_name',
+        'depends'
     )
 
     def __init__(self,
                  name: str = None, type_: Type = None,
-                 default: Any = Undefined, required: bool = Undefined, readonly: bool = Undefined,
+                 default: Any = Undefined, required: bool = Undefined, editable: bool = Undefined,
                  searchable: Comparator = Undefined, sortable: bool = Undefined,
                  format_: Format = None, icon: str = None, title: str = None, description: str = None, unit: str = None,
-                 source_field_name: str = None):
+                 source_field_name: str = None, depends: Any = None):
         """ Init method.
 
         :param type_: annotaion for field, e.g, str or Dict[str, str] or List[Object]
@@ -282,11 +284,11 @@ class ModelField:
             self.required = True
         else:
             self.required = required
-        # readonly is false if undefined
-        if readonly is Undefined:
-            self.readonly = False
+        # editable is true if undefined
+        if editable is Undefined:
+            self.editable = True
         else:
-            self.readonly = readonly
+            self.editable = editable
         # searchable is none if undefined
         if searchable is Undefined:
             self.searchable = None
@@ -305,6 +307,11 @@ class ModelField:
         self.unit = unit
         #
         self.source_field_name = source_field_name
+        # depends is mainly used to return dynamic data for page rendering and biz logic
+        # e.g,
+        # A tag field defiend in User model, tag: str = ModelField(format_=Format.SELECT, depends=lamdab self: self.team.tags)
+        # In page rendering, you can use user.tag_depends to draw select options
+        self.depends = depends
 
     def __str__(self):
         return f'{self.name}/{self.type}/{self.default}/{self.required}/{self.format}'
@@ -336,7 +343,7 @@ class RelationField(ModelField):
                  **kwargs):
         """ Init method.
 
-        :param save_field_name: The name to use for saving current relation, if it is none, will use xxx_id/xxx_ids by default. THe back field in related model have same value, used for search the original model
+        :param save_field_name: The name to use for saving current relation, if it is none, will use xxx_id/xxx_ids by default. The back field in related model have same value, used for search the original model
         :param save_field_order: If the field is List, use this order when loading and saving
         :param back_field_name: The name to use for the relation from the related model back to this one, if it is none, do not create back field in related model
         :param back_field_is_list: If back field is list
@@ -358,45 +365,6 @@ class RelationField(ModelField):
         self.is_back_field = is_back_field
 
 
-# ----------------------------------------------------------------------------------------------------------------------
-# Relation
-#
-
-class relation(object):
-    """ Decorator on a method to tell that it is used to fetch related models.
-
-    class Post(BaseModel):
-        creator_id: ObjectId
-        tag_ids: List[ObjectId]
-
-        @relation
-        def creator(self):
-            return User.find_one(self.creator_id)
-
-        @relation
-        def tags(self):
-            return list(Tag.find({'_id':{'$in':self.tag_ids}})
-
-    p = Post()
-    p.creator
-    p.tags
-
-    Reference:
-    https://docs.python.org/3/howto/descriptor.html#properties
-    https://realpython.com/primer-on-python-decorators/#classes-as-decorators
-    https://docs.python.org/3/reference/datamodel.html#object.__get__
-    """
-
-    def __init__(self, func):
-        functools.update_wrapper(self, func)
-        self.func = func
-
-    def __get__(self, obj, objtype=None):
-        if obj is None:
-            return self
-        return self.func(obj)
-
-
 # ----------------------------------------------------------------------------------------------------------------------
 # Meta class
 #
@@ -420,7 +388,7 @@ class ModelMeta(ABCMeta):
         :param namespace: Only contains the fields of current object
         """
         fields: Dict[str, ModelField] = {}
-        relations: Dict[str, relation] = {}
+        relations: Dict[str, RelationField] = {}
         properties: Dict[str, property] = {}
         # TODO: Support callable validators
         slots: Set[str] = namespace.get('__slots__', ())
@@ -537,16 +505,17 @@ class ModelMeta(ABCMeta):
                         name=field.save_field_name,
                         type_=List[id_type] if f_origin is list else id_type,
                         required=field.required,
+                        editable=False,  # save field value will be updated by its source field, so no need to render in form
                         source_field_name=ann_name,  # Mark the source field
                     )
-                    # Set required to false
-                    field.required = False
                     # back_field_name should always has a meaningful value, if it is none, do not create back field in related object
                     # e.g,
                     # project.activities.user -> user, do not need to create back_field in user model as it is hard to search all the activities for user
                     # in such case, we need to create activity model, activity.project -> project.activities, activity.user -> user.activities
                     if field.back_field_name is None:
                         pass
+                    #
+                    relations[ann_name] = value
                 # Define a ModelField
                 elif isinstance(value, ModelField):
                     # Supplement field's name and type, as we use annotation to define a model field
@@ -570,16 +539,20 @@ class ModelMeta(ABCMeta):
                             pass
                         elif not isinstance(field.default, ann_type):
                             raise SchemaError(f'{ann_name}: {ann_type} default value is invalid')
+                # check depends
+                if field.depends is not None:
+                    if callable(field.depends):
+                        pass
+                    elif not isinstance(field.depends, list):
+                        raise SchemaError(f'{ann_name}: {ann_type} depdends value should be list')
                 #
                 fields[ann_name] = field
                 if save_field is not None:
                     fields[save_field.name] = save_field
             #
-            # Relations & Properties
+            # Properties
             #
             for attr in namespace:
-                if isinstance(namespace.get(attr), relation):
-                    relations[attr] = namespace.get(attr)
                 if isinstance(namespace.get(attr), property):
                     properties[attr] = namespace.get(attr)
         #
@@ -611,11 +584,20 @@ class ModelMeta(ABCMeta):
                 # e.g,
                 # if use check_type.__fields__, when checking project.members, will fall into user.team and team.members recursion
                 # because the import mechanism, it is not possible to make explicit reference between user and team, so using annotations can prevent the recursion
-                for a_n, a_t in check_type.__dict__.get('__annotations__', {}).items():
-                    _iter(a_n, a_t, level + (1 if is_in_list else 0))
+                for c_n, c_t in check_type.__dict__.get('__annotations__', {}).items():
+                    c_f = check_type.__fields__[c_n]
+                    # Skip relation field
+                    if isinstance(c_f, RelationField):
+                        continue
+                    #
+                    _iter(c_n, c_t, level + (1 if is_in_list else 0))
 
         #
         for f in fields.values():
+            # Skip relation field
+            if isinstance(f, RelationField):
+                continue
+            #
             _iter(f.name, f.type, 0)
         #
         if max_list_depth >= 3:
@@ -660,7 +642,8 @@ class ModelMeta(ABCMeta):
                 back_field = RelationField(
                     name=f.back_field_name, type_=List[cls] if f.back_field_is_list else cls,
                     save_field_name=f.save_field_name, save_field_order=f.back_field_order,
-                    required=False, readonly=True,  # Relation field is lazy loaded, so it is not required
+                    required=False,  # back field is used for display only, so it is not required
+                    editable=False,  # ditto
                     format_=f.back_field_format, icon=f.back_field_icon, title=f.back_field_title, description=f.back_field_description,
                     unit=f.back_field_unit,
                     is_back_field=True,  # Mark this field to be a back field created by a relation field
@@ -675,6 +658,7 @@ class ModelMeta(ABCMeta):
                 # Update related model
                 f_type.__fields__[f.back_field_name] = back_field
                 f_type.__slots__ = tuple(f_type.__slots__) + (f.back_field_name,)
+                f_type.__relations__[f.back_field_name] = back_field
 
         #
         return cls
@@ -746,21 +730,26 @@ class BaseModel(metaclass=ModelMeta):
         # Validate against schema
         # print(f'Validate {cls.__name__} with {data}')
         for field_name, field_type in cls.__fields__.items():
-            # Skip relation field's validation
+            # Skip recursive validation in relation value, just keep the relation value
+            # please note: in save logic, we need to skip relation field, e.g, in mongosupport, we use self.dict(include_relations=False)
             if isinstance(field_type, RelationField):
+                relation_value = data.get(field_name, Undefined)
+                if relation_value is not Undefined:
+                    data[field_name] = relation_value
+                #
                 continue
             #
             # Update id/ids field created by relation field
-            # But can not do update in back relation field
+            # But can NOT do update in back relation field
             # e.g,
             # user.team -> team.members, team_id is saved in user model, we can use user.team = another team to update it
-            # However, it is not supported to update by team.members.append(user), because it is complex to implement this:
-            #   Remember which users are removed and appended from team
-            #   Add transaction support to update these users before updating team object
+            # However, it is not supported to update by team.members.append(user), because it is complex to implement below:
+            #   - Need to implement some logic to remember which users are removed and appended from team, especially there are huge amount of members
+            #   - Add transaction support to update these users before updating team object
             #
             if field_type.source_field_name is not None:  # e.g, team_id is created by team relation, so source field name of team_id is team
                 source_field = cls.__fields__[field_type.source_field_name]
-                relation_value = data.get(source_field.name, Undefined)  # get team value
+                relation_value = data.get(source_field.name, Undefined)  # e.g, get team value
                 # RelationField's value is lazy loaded, undefined means it is not load, so trust the value in current field
                 if relation_value is Undefined:
                     pass
@@ -781,7 +770,7 @@ class BaseModel(metaclass=ModelMeta):
                                     id_ = relation_type.__id_type__(id_)
                                 #
                                 update_value.append(id_)
-                    else:
+                    elif relation_value is not None:
                         relation_type = source_field.type
                         if isinstance(relation_value, dict):
                             id_ = relation_value.get(relation_type.__id_name__)
@@ -804,7 +793,7 @@ class BaseModel(metaclass=ModelMeta):
         # Check if any non-defined fields
         undefined_fields = set(data.keys()) - set(cls.__fields__.keys())
         if len(undefined_fields) > 0:
-            # Do not raise error for non-defined fields
+            # Do not raise error for non-defined fields, because we are using __dict__ to store data and other dynamic attributes also use __dict__ to store
             # errors.append(f'{cls.__name__}: Found undefined data fields, {undefined_fields}')
             pass
         #
@@ -917,6 +906,21 @@ class BaseModel(metaclass=ModelMeta):
         Note: When an attribute is accessed, __getattribute__ is invoked firstly
         and then if the attribute wasn't found, __getattr__ should be invoked.
         """
+        # Return depends data defined in a field, which can be function to return dynamic data
+        if name.endswith('_depends'):
+            name = name.split('_depends')[0]
+            field = self.__class__.__fields__.get(name)
+            if field and field.depends:
+                depends = field.depends
+                if callable(depends):
+                    # check if depends function has arguments
+                    if depends.__code__.co_argcount == 0:
+                        return depends()
+                    else:
+                        return depends(self)  # Can only have one argument at most
+                else:
+                    return depends
+        #
         if name in self.__class__.__fields__:
             field = self.__class__.__fields__[name]
             f_type = field.type
@@ -979,11 +983,7 @@ class BaseModel(metaclass=ModelMeta):
                     self.__setattr__(name, default)
             #
             return default
-        # Because __dict__ is overwrited by values, so we need invoke relation mannaully
-        if name in self.__class__.__relations__:
-            rel = self.__class__.__relations__[name]
-            return rel.__get__(self)
-        # Invoke property mannaully
+        # Because __dict__ is overwrited by values, so we need to invoke property mannaully
         if name in self.__class__.__properties__:
             prp = self.__class__.__properties__[name]
             return prp.__get__(self)
@@ -1118,12 +1118,12 @@ class BaseModel(metaclass=ModelMeta):
     @classmethod
     def find_one(cls, filter_or_id, *args, **kwargs):
         """ abstract method, use it to fetch one related model. """
-        raise NotImplementedError()
+        return None
 
     @classmethod
     def find(cls, *args, **kwargs):
         """ abstract method, use it to fetch many related models. """
-        raise NotImplementedError()
+        return []
 
     @classmethod
     def schema(cls):
@@ -1135,13 +1135,13 @@ class BaseModel(metaclass=ModelMeta):
         https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.3.md#schemaObject
         https://swagger.io/docs/specification/data-models/
 
-        However, we still have some grammars
+        However, we still have some grammars, e.g,
           - add type date
-          - add enum_titles, py_type, layout, form, read, icon, readonly to help code generation
-          - Add format to array, so that we can gen a component for the whole array
-          - Add searchables to object, so that it can be used to generate search form
-          - Add sortables to object, so that it can be used to generate order drowpdown
-          - Add columns to object, so that it can be used to generate columns for table
+          - add enum_titles, py_type, layout, form, read, icon, editable to help code generation
+          - add format to array, so that we can gen a component for the whole array
+          - add searchables to object, so that it can be used to generate search form
+          - add sortables to object, so that it can be used to generate order drowpdown
+          - add columns to object, so that it can be used to generate columns for table
         """
 
         def _gen_schema(type_: Type, parents=[]):
@@ -1162,28 +1162,31 @@ class BaseModel(metaclass=ModelMeta):
                 #   User.team:Team -> Team.members:List[User] == List[#User]
                 check_parents = [type_.__name__] + parents
                 #
-                properties = {}
-                required, searchables, sortables, relations = [], [], [], []
+                properties = {}  # {field_name:field_schema}
+                requires, searchables, sortables = [], [], []
+                current_relations, current_form_relations = {}, {}  # {model_name:[field_name]}, use dict to keep the order
+                inner_relations, inner_form_relations = {}, {}  # {model_name:None}, ditto
                 for f_n, f_t in type_.__fields__.items():
                     field_schema = {}
-                    origin = get_origin(f_t.type)
+                    f_origin = get_origin(f_t.type)
+                    inner_type = None
                     # Dict
-                    if origin is dict:
+                    if f_origin is dict:
+                        f_type = get_args(f_t.type)[1]
                         # TODO: SUPPORT DICT
-                        pass
                     # List
-                    elif origin is list:
-                        l_type = get_args(f_t.type)[0]
-                        if l_type.__name__ in check_parents:
+                    elif f_origin is list:
+                        f_type = get_args(f_t.type)[0]
+                        if f_type.__name__ in check_parents:
                             field_schema.update({
                                 'type': 'array',
                                 # https://json-schema.org/understanding-json-schema/structuring.html#recursion
                                 # Your program should replace the {$ref:} with reference object's schema
-                                'items': {'$ref': f'#{l_type.__name__}'},
-                                'py_type': f'List[{type_.__name__}]',
+                                'items': {'$ref': f'#{f_type.__name__}'},
+                                'py_type': f'List[{f_type.__name__}]',
                             })
                         else:
-                            inner_type = _gen_schema(l_type, check_parents)
+                            inner_type = _gen_schema(f_type, check_parents)
                             field_schema.update({
                                 'type': 'array',
                                 'items': inner_type,
@@ -1191,16 +1194,17 @@ class BaseModel(metaclass=ModelMeta):
                             })
                     # built-in type, SimpleEnum or sub model
                     else:
-                        if f_t.type.__name__ in check_parents:
+                        f_type = f_t.type
+                        if f_type.__name__ in check_parents:
                             field_schema.update({
                                 'type': 'object',
                                 # Your program should replace the {$ref:} with properties of reference object's schema only
                                 # But should NOT overwrite the icon/title/description/... as these fields may use different values
-                                '$ref': f'#{f_t.type.__name__}',
-                                'py_type': f_t.type.__name__,
+                                '$ref': f'#{f_type.__name__}',
+                                'py_type': f_type.__name__,
                             })
                         else:
-                            inner_type = _gen_schema(f_t.type, check_parents)
+                            inner_type = _gen_schema(f_type, check_parents)
                             field_schema.update(inner_type)
                     # default
                     if f_t.default:
@@ -1230,29 +1234,41 @@ class BaseModel(metaclass=ModelMeta):
                     # required
                     if f_t.required:
                         field_schema.update({'required': True})
-                        required.append(f_n)
-                    # readonly
-                    if f_t.readonly:
-                        field_schema.update({'readonly': True})
+                        requires.append(f_n)
+                    # editable
+                    field_schema.update({'editable': f_t.editable})
                     # searchable
                     if f_t.searchable is not None:
                         searchables.append((f_n, f_t.searchable))
                     # sortable
                     if f_t.sortable is True:
                         sortables.append(f_n)
+                    # depends, only update it to true, the you can access it by {field_name}_depends in page rendering or biz logic
+                    if f_t.depends:
+                        field_schema.update({'depends': True})
                     # relation
                     if isinstance(f_t, RelationField):
                         field_schema.update({'is_relation': True})
+                        field_schema.update({'is_out_relation': not f_t.is_back_field})
                         field_schema.update({'is_back_relation': f_t.is_back_field})
-                        relations.append(f_n)
+                        field_schema.update({'save_field_name': f_t.save_field_name})
+                        # relations contains all the related model names, do not include any back relations
+                        if not f_t.is_back_field:
+                            current_relations.setdefault(f_type.__name__, [])
+                            current_relations[f_type.__name__] += [f_n]
+                            if inner_type and 'relations' in inner_type:
+                                inner_relations.update({k: None for k in inner_type['relations']})
+                                inner_form_relations.update({k: None for k in inner_type['form_relations']})
+
                     #
                     properties[f_n] = field_schema
                 #
                 obj = {
                     'type': 'object',
                     'properties': properties,
-                    'required': required,
-                    'columns': type_.__columns__ if type_.__columns__ else required,  # Using required fields as columns as the default
+                    'requires': requires,
+                    'editable': True,
+                    'columns': type_.__columns__ if type_.__columns__ else requires,  # Using required fields as columns as the default
                     'py_type': type_.__name__,
                     'icon': type_.__icon__ if type_.__icon__ else None,
                     'title': type_.__title__ if type_.__title__ else type_.__name__.upper(),
@@ -1264,18 +1280,23 @@ class BaseModel(metaclass=ModelMeta):
                 obj['read'] = parse_layout(type_.__read__)[0] if type_.__read__ else layout
                 obj['form'] = parse_layout(type_.__form__)[0] if type_.__form__ else layout
                 obj['groups'] = [parse_layout(g)[0] for g in type_.__groups__]
-                # each column in layout can be blank('')/summary($)/hyphen(-)/group(number)/field(string) suffixed with query and span string
-                # read_fields/form_fields just return field names
+                # each column in layout can be blank('')/hyphen(-)/summary($)/group(number)/field(string)/inner fields(has children) suffixed with query and span string
+                # read_fields/form_fields just return valid field names
                 obj['read_fields'] = list(iterate_layout(obj['read'], obj['groups']))
                 obj['form_fields'] = list(iterate_layout(obj['form'], obj['groups']))
-                # searchables
+                # searchables, [field__comparator]
                 obj['searchables'] = [('{}__{}'.format(*s) if s[1] != Comparator.EQ else s[0]) for s in searchables]
-                # sortables
+                obj['search_fields'] = [s[0] for s in searchables]
+                # sortables, [(field_name, 1/-1)]
                 obj['sortables'] = sortables
-                # relations
-                obj['relations'] = relations
-                # id_name
+                # relations recrusively
+                inner_relations.update(current_relations)
+                obj['relations'] = [k for k in inner_relations.keys() if k != type_.__name__]  # skip self because inner_relations may contains it
+                inner_form_relations.update({k: None for k, v in current_relations.items() if any((True for vv in v if vv in obj['form_fields']))})
+                obj['form_relations'] = [k for k in inner_form_relations.keys() if k != type_.__name__]
+                # id_name & id_type, used for relation inputs
                 obj['id_name'] = type_.__id_name__
+                obj['id_type'] = type_.__id_type__.__name__ if type_.__id_type__ else None
                 return obj
             elif type_ is str:
                 return {'type': 'string', 'format': Format.TEXT, 'py_type': 'str'}
