@@ -61,15 +61,19 @@ def generate_names(name):
         }
     else:
         name_hyphen = re.sub(r'[.,?=#+]', '-', ''.join(name.split()))  # e.g, plan.members-form -> plan-members-form
+        name_snake = inflection.underscore(name_hyphen)
+        name_kebab = inflection.dasherize(name_snake)
+        name_title = inflection.titleize(name_hyphen)
         return {
             'name': name,  # => SampleModel
             'name_lower': name.lower(),  # => samplemodel
-            'name_kebab': inflection.dasherize(inflection.underscore(name_hyphen)),  # => sample-model
-            'name_camel': inflection.camelize(name_hyphen, uppercase_first_letter=False),  # => sampleModel
-            'name_snake': inflection.underscore(name_hyphen),  # => sample_model
-            'name_snake_plural': inflection.tableize(name_hyphen),  # => sample_models
-            'name_title': inflection.titleize(name_hyphen),  # => Sample Model
-            'name_title_lower': inflection.titleize(name_hyphen).lower(),  # => sample model
+            'name_snake': name_snake,  # => sample_model
+            'name_snake_plural': inflection.pluralize(name_snake),  # => sample_models
+            'name_kebab': name_kebab,  # => sample-model
+            'name_kebab_plural': inflection.pluralize(name_kebab),  # => sample-models
+            'name_title': name_title,  # => Sample Model
+            'name_title_lower': name_title.lower(),  # => sample model
+            'name_title_lower_plural': inflection.pluralize(name_title.lower()),  # => sample models
         }
 
 
